/**
 * @file
 * @brief CAN message filter types.
 */
#ifndef STA_CORE_CAN_FILTER_HPP
#define STA_CORE_CAN_FILTER_HPP

#include <sta/bus/can/id.hpp>

#include <cstdint>


namespace sta
{
    /**
     * @defgroup sta_core_can_filters Filters
     * @ingroup sta_core_can
     * @brief CAN message filter types.
     * @{
     */


    /**
     * @brief ID format matched by CAN filter.
     */
    enum class CanFilterIdFormat
    {
        ANY,    /**< Match both ID formats */
        STD,    /**< Match standard format IDs */
        EXT     /**< Match extended format IDs */
    };

    /**
     * @brief CAN filter settings.
     */
    struct CanFilter
    {
        CanId             obj;  /**< ID object */
        CanId             mask; /**< ID mask */
        CanFilterIdFormat type; /**< ID format to match */
        uint8_t           fifo; /**< FIFO to store matches */
    };


    /** @} */
} // namespace sta


#endif // STA_CORE_CAN_FILTER_HPP
